/****************************************************************************
**
** Copyright (C) 2023 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt Design Viewer of the Qt Toolkit.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include <QQuickView>

class ProjectManager : public QObject
{
    Q_OBJECT
public:
    explicit ProjectManager(QObject *parent = nullptr);
    ~ProjectManager();

    bool isRunning();

public slots:
    bool runProject(const QByteArray &project, const bool autoScaleProject);
    void stopProject();

private:
    // Member variables
    QByteArray m_projectData;
    QString m_projectPath;
    bool m_autoScaleProject;

    // Qml related members
    QScopedPointer<QQmlEngine> m_qmlEngine;
    QScopedPointer<QQmlComponent> m_qmlComponent;
    QScopedPointer<QQuickWindow> m_quickWindow;

    // resource management
    bool registerResource(const QByteArray &data, const QString &path);
    bool unregisterResource(const QByteArray &data, const QString &path);
    bool copyResourceToFs(const QString &sourcePath, const QString &targetPath);
    QString unpackProject(const QByteArray &project);

    // project parsers
    QString findFile(const QString &dir, const QString &filter);
    QString readQmlProjectFile(const QString &qmlProjectFilePath);
    QString getMainQmlFile(const QString &projectPath, const QString &qmlProjectFileContent);
    QStringList getImportPaths(const QString &projectPath, const QString &qmlProjectFileContent);
    bool isQt6Project(const QString &qmlProjectFileContent);
    void parseQmlProjectFile(const QString &fileName, QString *mainFile, QStringList *importPaths);

    // window management
    void orientateWindow(Qt::ScreenOrientation orientation);
    void showAppWindow();

signals:
    void closingProject();
};
